@@ -41,6 +41,8 @@ private Q_SLOTS:
     void shouldParseVcard3();
     void shouldParseVcard4();
 
+    void shouldExportVCard21();
+    void shouldParseVCard21();
 };
 
 #endif
