@@ -161,7 +161,7 @@ public:
     static KContacts::SortMode *mSortMode;
 };
 
-KContacts::SortMode *Addressee::Private::mSortMode = Q_NULLPTR;
+KContacts::SortMode *Addressee::Private::mSortMode = nullptr;
 
 Addressee::Addressee()
     : d(new Private)
@@ -1471,7 +1471,7 @@ void Addressee::setNameFromString(const QString &s)
         int leftOffset = 0;
         int rightOffset = parts.count() - 1;
 
-        if (parts.count() > 0) {
+        if (!parts.isEmpty()) {
 
             QString suffix;
             while (rightOffset >= 0) {
@@ -1796,7 +1796,7 @@ PhoneNumber::List Addressee::phoneNumbers(PhoneNumber::Type type) const
     PhoneNumber::List list;
 
     PhoneNumber::List::ConstIterator it;
-    PhoneNumber::List::ConstIterator end(d->mPhoneNumbers.constEnd());
+    const PhoneNumber::List::ConstIterator end(d->mPhoneNumbers.constEnd());
     for (it = d->mPhoneNumbers.constBegin(); it != end; ++it) {
         if (matchBinaryPattern((*it).type(), type)) {
             list.append(*it);
@@ -1808,7 +1808,7 @@ PhoneNumber::List Addressee::phoneNumbers(PhoneNumber::Type type) const
 PhoneNumber Addressee::findPhoneNumber(const QString &id) const
 {
     PhoneNumber::List::ConstIterator it;
-    PhoneNumber::List::ConstIterator end(d->mPhoneNumbers.constEnd());
+    const PhoneNumber::List::ConstIterator end(d->mPhoneNumbers.constEnd());
     for (it = d->mPhoneNumbers.constBegin(); it != end; ++it) {
         if ((*it).id() == id) {
             return *it;
@@ -2191,7 +2191,7 @@ void Addressee::setCustoms(const QStringList &customs)
 
     d->mCustomFields.clear();
 
-    foreach (const QString &custom, customs) {
+    for (const QString &custom : customs) {
         const int index = custom.indexOf(QLatin1Char(':'));
         if (index == -1) {
             continue;
@@ -2208,11 +2208,12 @@ QStringList Addressee::customs() const
 {
     QStringList result;
 
-    QHashIterator<QString, QString> it(d->mCustomFields);
+    QHash<QString, QString>::const_iterator it = d->mCustomFields.constBegin();
+    const QHash<QString, QString>::const_iterator end = d->mCustomFields.constEnd();
     result.reserve(d->mCustomFields.count());
-    while (it.hasNext()) {
-        it.next();
+    while (it != end) {
         result << it.key() + QLatin1Char(':') + it.value();
+        ++it;
     }
 
     return result;
