@@ -49,6 +49,7 @@
 #include "nickname.h"
 #include "role.h"
 #include "org.h"
+#include "clientpidmap.h"
 
 namespace KContacts
 {
@@ -267,6 +268,10 @@ public:
       Set nick name.
      */
     void setNickName(const QString &nickName);
+    void setNickName(const NickName &nickName);
+    void insertExtraNickName(const NickName &nickName);
+    void setExtraNickNameList(const NickName::List &nickNameList);
+    NickName::List extraNickNameList() const;
 
     /**
       Return nick name.
@@ -279,15 +284,28 @@ public:
     static QString nickNameLabel();
 
     /**
-      Set birthday.
+      Set birthday (date and time). If withTime is false the time will be set
+      to midnight and birthdayHasTime() will return false afterwards.
+      @since 5.4
      */
-    void setBirthday(const QDateTime &birthday);
+    void setBirthday(const QDateTime &birthday, bool withTime = true);
 
     /**
-      Return birthday.
+      Set birthday (date only). birthdayHasTime() will return false afterwards.
+     */
+    void setBirthday(const QDate &birthday);
+
+    /**
+      Return birthday. (If a valid date has been set, birthday().time() will
+      always return a valid QTime!)
      */
     QDateTime birthday() const;
 
+    /**
+      Returns true if birthday has been set with a time. Returns false otherwise.
+     */
+    bool birthdayHasTime() const;
+
     /**
       Return translated label for birthday field.
      */
@@ -501,7 +519,6 @@ public:
     void setExtraOrganizationList(const Org::List &orgList);
     Org::List extraOrganizationList() const;
 
-
     /**
       Return organization.
      */
@@ -1062,6 +1079,11 @@ public:
     void setFieldGroupList(const FieldGroup::List &fieldGroupList);
     void insertFieldGroup(const FieldGroup &fieldGroup);
 
+    //ClientPidMap
+    ClientPidMap::List clientPidMapList() const;
+    void setClientPidMapList(const ClientPidMap::List &clientpidmaplist);
+    void insertClientPidMap(const ClientPidMap &clientpidmap);
+
 private:
     class Private;
     QSharedDataPointer<Private> d;
