@@ -1,7 +1,7 @@
 /*
     This file is part of the KContacts framework.
     Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
-    Copyright (c) 2015-2016 Laurent Montel <montel@kde.org>
+    Copyright (c) 2015-2017 Laurent Montel <montel@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -49,30 +49,67 @@ static bool needsEncoding(const QString &value)
     return false;
 }
 
+static const struct {
+    const char *addressType;
+    Address::TypeFlag flag;
+} s_addressTypes[] = {
+    { "dom", Address::Dom },
+    { "home", Address::Home },
+    { "intl", Address::Intl },
+    { "parcel", Address::Parcel },
+    { "postal", Address::Postal },
+    { "pref", Address::Pref },
+    { "work", Address::Work },
+};
+
+static const unsigned int s_numAddressTypes
+    = sizeof s_addressTypes / sizeof * s_addressTypes;
+
+static Address::TypeFlag stringToAddressType(const QString &str)
+{
+    for (unsigned int i = 0; i < s_numAddressTypes; ++i)
+        if (str == QLatin1String(s_addressTypes[i].addressType)) {
+            return s_addressTypes[i].flag;
+        }
+    return {};
+}
+
+static const struct {
+    const char *phoneType;
+    PhoneNumber::TypeFlag flag;
+} s_phoneTypes[] = {
+    { "BBS", PhoneNumber::Bbs },
+    { "CAR", PhoneNumber::Car },
+    { "CELL", PhoneNumber::Cell },
+    { "FAX", PhoneNumber::Fax },
+    { "HOME", PhoneNumber::Home },
+    { "ISDN", PhoneNumber::Isdn },
+    { "MODEM", PhoneNumber::Modem },
+    { "MSG", PhoneNumber::Msg },
+    { "PAGER", PhoneNumber::Pager },
+    { "PCS", PhoneNumber::Pcs },
+    { "PREF", PhoneNumber::Pref },
+    { "VIDEO", PhoneNumber::Video },
+    { "VOICE", PhoneNumber::Voice },
+    { "WORK", PhoneNumber::Work },
+};
+
+static const unsigned int s_numPhoneTypes
+    = sizeof s_phoneTypes / sizeof * s_phoneTypes;
+
+static PhoneNumber::TypeFlag stringToPhoneType(const QString &str)
+{
+    for (unsigned int i = 0; i < s_numPhoneTypes; ++i) {
+        if (str == QLatin1String(s_phoneTypes[i].phoneType)) {
+            return s_phoneTypes[i].flag;
+        }
+    }
+    return {};
+}
+
 VCardTool::VCardTool()
 {
-    mAddressTypeMap.insert(QStringLiteral("dom"), Address::Dom);
-    mAddressTypeMap.insert(QStringLiteral("intl"), Address::Intl);
-    mAddressTypeMap.insert(QStringLiteral("postal"), Address::Postal);
-    mAddressTypeMap.insert(QStringLiteral("parcel"), Address::Parcel);
-    mAddressTypeMap.insert(QStringLiteral("home"), Address::Home);
-    mAddressTypeMap.insert(QStringLiteral("work"), Address::Work);
-    mAddressTypeMap.insert(QStringLiteral("pref"), Address::Pref);
 
-    mPhoneTypeMap.insert(QStringLiteral("HOME"), PhoneNumber::Home);
-    mPhoneTypeMap.insert(QStringLiteral("WORK"), PhoneNumber::Work);
-    mPhoneTypeMap.insert(QStringLiteral("MSG"), PhoneNumber::Msg);
-    mPhoneTypeMap.insert(QStringLiteral("PREF"), PhoneNumber::Pref);
-    mPhoneTypeMap.insert(QStringLiteral("VOICE"), PhoneNumber::Voice);
-    mPhoneTypeMap.insert(QStringLiteral("FAX"), PhoneNumber::Fax);
-    mPhoneTypeMap.insert(QStringLiteral("CELL"), PhoneNumber::Cell);
-    mPhoneTypeMap.insert(QStringLiteral("VIDEO"), PhoneNumber::Video);
-    mPhoneTypeMap.insert(QStringLiteral("BBS"), PhoneNumber::Bbs);
-    mPhoneTypeMap.insert(QStringLiteral("MODEM"), PhoneNumber::Modem);
-    mPhoneTypeMap.insert(QStringLiteral("CAR"), PhoneNumber::Car);
-    mPhoneTypeMap.insert(QStringLiteral("ISDN"), PhoneNumber::Isdn);
-    mPhoneTypeMap.insert(QStringLiteral("PCS"), PhoneNumber::Pcs);
-    mPhoneTypeMap.insert(QStringLiteral("PAGER"), PhoneNumber::Pager);
 }
 
 VCardTool::~VCardTool()
@@ -101,7 +138,7 @@ void VCardTool::addParameters(VCardLine &line, const QMap<QString, QStringList>
 void VCardTool::addParameter(VCardLine &line, VCard::Version version, const QString &key, const QStringList &valueStringList) const
 {
     if (version == VCard::v2_1) {
-        Q_FOREACH (const QString &valueStr, valueStringList) {
+        for (const QString &valueStr : valueStringList) {
             line.addParameter(valueStr, QString());
         }
     } else if (version == VCard::v3_0) {
@@ -177,15 +214,14 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             }
 
             const bool hasLabel = !(*it).label().isEmpty();
-            QMap<QString, Address::TypeFlag>::ConstIterator typeIt;
             QStringList addreLineType;
             QStringList labelLineType;
-            for (typeIt = mAddressTypeMap.constBegin();
-                    typeIt != mAddressTypeMap.constEnd(); ++typeIt) {
-                if (typeIt.value() & (*it).type()) {
-                    addreLineType << typeIt.key();
+            for (unsigned int i = 0; i < s_numAddressTypes; ++i) {
+                if (s_addressTypes[i].flag & (*it).type()) {
+                    const QString str = QString::fromLatin1(s_addressTypes[i].addressType);
+                    addreLineType << str;
                     if (hasLabel) {
-                        labelLineType << typeIt.key();
+                        labelLineType << str;
                     }
                 }
             }
@@ -244,14 +280,16 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         if (version == VCard::v4_0) {
             // The KIND property must be set to "group" in order to use this property.
             if ((*addrIt).kind().toLower() == QLatin1String("group")) {
-                Q_FOREACH (const QString &member, (*addrIt).members()) {
+                const QStringList lst = (*addrIt).members();
+                for (const QString &member : lst) {
                     VCardLine line(QStringLiteral("MEMBER"), member);
                     card.addLine(line);
                 }
             }
         }
         // SOURCE
-        Q_FOREACH (const QUrl &url, (*addrIt).sourcesUrlList()) {
+        const QVector<QUrl> lstUrl = (*addrIt).sourcesUrlList();
+        for (const QUrl &url : lstUrl) {
             VCardLine line = VCardLine(QStringLiteral("SOURCE"), url.url());
             card.addLine(line);
         }
@@ -303,8 +341,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                     if (i.key().toLower() == QLatin1String("type")) {
                         QStringList valueStringList = i.value();
                         bool hasPreferred = false;
-                        if (valueStringList.contains(QStringLiteral("PREF"))) {
-                            valueStringList.removeAll(QStringLiteral("PREF"));
+                        const int removeItems = valueStringList.removeAll(QStringLiteral("PREF"));
+                        if (removeItems > 0) {
                             hasPreferred = true;
                         }
                         if (!valueStringList.isEmpty()) {
@@ -353,7 +391,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
         // LOGO
         card.addLine(createPicture(QStringLiteral("LOGO"), (*addrIt).logo(), version));
-        Q_FOREACH (const Picture &logo, (*addrIt).extraLogoList()) {
+        const QVector<Picture> lstLogo = (*addrIt).extraLogoList();
+        for (const Picture &logo : lstLogo) {
             card.addLine(createPicture(QStringLiteral("LOGO"), logo, version));
         }
 
@@ -407,7 +446,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
         // NICKNAME only for version > 2.1
         if (version != VCard::v2_1) {
-            Q_FOREACH (const NickName &nickName, (*addrIt).extraNickNameList()) {
+            const QVector<NickName> lstNickName = (*addrIt).extraNickNameList();
+            for (const NickName &nickName : lstNickName) {
                 VCardLine nickNameLine(QStringLiteral("NICKNAME"), nickName.nickname());
                 addParameters(nickNameLine, nickName.parameters());
 
@@ -424,7 +464,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(noteLine);
 
         // ORG
-        Q_FOREACH (const Org &org, (*addrIt).extraOrganizationList()) {
+        const QVector<Org> lstOrg = (*addrIt).extraOrganizationList();
+        for (const Org &org : lstOrg) {
             QStringList organization;
             organization.append(org.organization().replace(QLatin1Char(';'),
                                 QStringLiteral("\\;")));
@@ -444,7 +485,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
         // PHOTO
         card.addLine(createPicture(QStringLiteral("PHOTO"), (*addrIt).photo(), version));
-        Q_FOREACH (const Picture &photo, (*addrIt).extraPhotoList()) {
+        const QVector<Picture> lstExtraPhoto = (*addrIt).extraPhotoList();
+        for (const Picture &photo : lstExtraPhoto) {
             card.addLine(createPicture(QStringLiteral("PHOTO"), photo, version));
         }
 
@@ -457,7 +499,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(VCardLine(QStringLiteral("REV"), createDateTime((*addrIt).revision(), version)));
 
         // ROLE
-        Q_FOREACH (const Role &role, (*addrIt).extraRoleList()) {
+        const QVector<Role> lstExtraRole = (*addrIt).extraRoleList();
+        for (const Role &role : lstExtraRole) {
             VCardLine roleLine(QStringLiteral("ROLE"), role.role());
             if (version == VCard::v2_1 && needsEncoding(role.role())) {
                 roleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
@@ -474,7 +517,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
 
         // SOUND
         card.addLine(createSound((*addrIt).sound(), version));
-        Q_FOREACH (const Sound &sound, (*addrIt).extraSoundList()) {
+        const QVector<Sound> lstSound = (*addrIt).extraSoundList();
+        for (const Sound &sound : lstSound) {
             card.addLine(createSound(sound, version));
         }
 
@@ -492,15 +536,14 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                 }
             }
 
-            QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeIt;
-            QMap<QString, PhoneNumber::TypeFlag>::ConstIterator typeEnd(mPhoneTypeMap.constEnd());
             QStringList lst;
-            for (typeIt = mPhoneTypeMap.constBegin(); typeIt != typeEnd; ++typeIt) {
-                if (typeIt.value() & (*phoneIt).type()) {
+            for (unsigned int i = 0; i < s_numPhoneTypes; ++i) {
+                if (s_phoneTypes[i].flag & (*phoneIt).type()) {
+                    const QString str = QString::fromLatin1(s_phoneTypes[i].phoneType);
                     if (version == VCard::v4_0) {
-                        lst << typeIt.key().toLower();
+                        lst << str.toLower();
                     } else {
-                        lst << typeIt.key();
+                        lst << str;
                     }
                 }
             }
@@ -511,7 +554,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         }
 
         // TITLE
-        Q_FOREACH (const Title &title, (*addrIt).extraTitleList()) {
+        const QVector<Title> lstTitle = (*addrIt).extraTitleList();
+        for (const Title &title : lstTitle) {
             VCardLine titleLine(QStringLiteral("TITLE"), title.title());
             if (version == VCard::v2_1 && needsEncoding(title.title())) {
                 titleLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
@@ -544,7 +588,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         card.addLine(VCardLine(QStringLiteral("UID"), (*addrIt).uid()));
 
         // URL
-        Q_FOREACH (const ResourceLocatorUrl &url, (*addrIt).extraUrlList()) {
+        const QVector<ResourceLocatorUrl> lstExtraUrl = (*addrIt).extraUrlList();
+        for (const ResourceLocatorUrl &url : lstExtraUrl) {
             VCardLine line(QStringLiteral("URL"), url.url());
             addParameters(line, url.parameters());
             card.addLine(line);
@@ -571,7 +616,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         }
         // From vcard4.
         if (version == VCard::v4_0) {
-            Q_FOREACH (const CalendarUrl &url, (*addrIt).calendarUrlList()) {
+            const QVector<CalendarUrl> lstCalendarUrl = (*addrIt).calendarUrlList();
+            for (const CalendarUrl &url : lstCalendarUrl) {
                 if (url.isValid()) {
                     QString type;
                     switch (url.type()) {
@@ -598,14 +644,16 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             }
         }
         //FieldGroup
-        Q_FOREACH (const FieldGroup &group, (*addrIt).fieldGroupList()) {
+        const QVector<FieldGroup> lstGroup = (*addrIt).fieldGroupList();
+        for (const FieldGroup &group : lstGroup) {
             VCardLine line(group.fieldGroupName(), group.value());
             addParameters(line, group.parameters());
             card.addLine(line);
         }
 
         // IMPP (supported in vcard 3 too)
-        Q_FOREACH (const Impp &impp, (*addrIt).imppList()) {
+        const QVector<Impp> lstImpp = (*addrIt).imppList();
+        for (const Impp &impp : lstImpp) {
             VCardLine line(QStringLiteral("IMPP"), impp.address());
             line.addParameter(QStringLiteral("X-SERVICE-TYPE"), Impp::typeToString(impp.type()));
             QMapIterator<QString, QStringList> i(impp.parameters());
@@ -754,7 +802,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     const QStringList types = (*lineIt).parameters(QStringLiteral("type"));
                     QStringList::ConstIterator end(types.end());
                     for (QStringList::ConstIterator it = types.begin(); it != end; ++it) {
-                        type |= mAddressTypeMap[(*it).toLower() ];
+                        type |= stringToAddressType((*it).toLower());
                     }
 
                     address.setType(type);
@@ -924,7 +972,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     const QStringList types = (*lineIt).parameters(QStringLiteral("type"));
                     QStringList::ConstIterator end(types.end());
                     for (QStringList::ConstIterator it = types.begin(); it != end; ++it) {
-                        type |= mAddressTypeMap[(*it).toLower() ];
+                        type |= stringToAddressType((*it).toLower());
                     }
 
                     bool available = false;
@@ -1071,7 +1119,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     const QStringList types = (*lineIt).parameters(QStringLiteral("type"));
                     QStringList::ConstIterator typeEnd(types.constEnd());
                     for (QStringList::ConstIterator it = types.constBegin(); it != typeEnd; ++it) {
-                        type |= mPhoneTypeMap[(*it).toUpper()];
+                        type |= stringToPhoneType((*it).toUpper());
                         foundType = true;
                     }
                     phone.setType(foundType ? type : PhoneNumber::Undefined);
@@ -1205,25 +1253,29 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
 QDateTime VCardTool::parseDateTime(const QString &str, bool *timeValid)
 {
-    const QStringList strings = str.split(QLatin1Char('T'));
-
-    QString dateString = strings.at(0);
+    const int posT = str.indexOf(QLatin1Char('T'));
+    QString dateString = posT >= 0 ? str.left(posT) : str;
     const bool noYear = dateString.startsWith(QLatin1String("--"));
-    dateString = dateString.replace(QLatin1String("-"), QString());
-    QDate date = QDate::fromString(dateString, QStringLiteral("yyyyMMdd"));
+    dateString = dateString.remove(QLatin1Char('-'));
+    QDate date;
     if (noYear) {
         date = QDate::fromString(dateString, QStringLiteral("MMdd"));
         date = date.addYears(-1900);
+    } else {
+        date = QDate::fromString(dateString, QStringLiteral("yyyyMMdd"));
     }
 
     QTime time;
     Qt::TimeSpec spec = Qt::LocalTime;
     int offsetSecs = 0;
-    if (strings.length() > 1) {
-        QString timeString = strings.at(1);
-        timeString = timeString.replace(QLatin1String(":"), QString());
-        QStringList timeStrings = timeString.split(QRegularExpression(QStringLiteral("[Z+-]")));
-        const QString hhmmssString = timeStrings.at(0);
+    if (posT >= 0) {
+        QString timeString = str.mid(posT + 1);
+        timeString = timeString.remove(QLatin1Char(':'));
+        const int zPos = timeString.indexOf(QLatin1Char('Z'));
+        const int plusPos = timeString.indexOf(QLatin1Char('+'));
+        const int minusPos = timeString.indexOf(QLatin1Char('-'));
+        const int tzPos = qMax(qMax(zPos, plusPos), minusPos);
+        const QString hhmmssString = tzPos >= 0 ? timeString.left(tzPos) : timeString;
         switch (hhmmssString.size()) {
         case 2:
             time = QTime::fromString(hhmmssString, QStringLiteral("hh"));
@@ -1235,13 +1287,13 @@ QDateTime VCardTool::parseDateTime(const QString &str, bool *timeValid)
             time = QTime::fromString(hhmmssString, QStringLiteral("hhmmss"));
             break;
         }
-        if (timeStrings.length() > 1) {
-            if (timeString.contains(QLatin1Char('Z'))) {
+        if (tzPos >= 0) {
+            if (zPos >= 0) {
                 spec = Qt::UTC;
             } else {
                 spec = Qt::OffsetFromUTC;
                 QTime offsetTime;
-                const QString offsetString = timeStrings.at(1);
+                const QString offsetString = timeString.mid(tzPos + 1);
                 switch (offsetString.size()) {
                 case 2:
                     offsetTime = QTime::fromString(offsetString, QStringLiteral("hh"));
@@ -1252,7 +1304,7 @@ QDateTime VCardTool::parseDateTime(const QString &str, bool *timeValid)
                 }
                 offsetSecs = offsetTime.hour() * 3600 + offsetTime.minute() * 60;
             }
-            if (timeString.contains(QLatin1Char('-'))) {
+            if (minusPos >= 0) {
                 offsetSecs *= -1;
             }
         }
